using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;
using ZennoLab.CommandCenter;
using ZennoLab.InterfacesLibrary;
using ZennoLab.InterfacesLibrary.ProjectModel;
using ZennoLab.InterfacesLibrary.ProjectModel.Collections;
using ZennoLab.InterfacesLibrary.ProjectModel.Enums;
using ZennoLab.Macros;
using Global.ZennoExtensions;
using ZennoLab.Emulation;
using System.Net.Http;
using System.Xml;
using Telegram.Bot.Args;
using Telegram.Bot;
using Telegram.Bot.Types.ReplyMarkups;
using Telegram.Bot.Types.InputFiles;
using System.Diagnostics;

namespace ZennoLab.OwnCode
{


	class TG
	{
		public string apikey = "";
		public TelegramBotClient client;
		public IZennoPosterProjectModel project;
		public static object tgLock = new object();
		string tasksRunerPath = Environment.CurrentDirectory + @"\TasksRunner.exe";


		// Список команд менять здесь
		private const string cAddTries = "Добавить попыток";
		private const string cInterruptTask = "Прервать";
		private const string cSetTries = "Установить попыток";
		private const string cSetThreads = "Максимум потоков";
		private const string cStartTask = "Запустить";
		private const string cStopTask = "Стоп";
		private const string cGetCpu = "Цпу";
		private const string cAll = "/all";
		private const string cWorking = "/working";
		private const string cHelp = "/help";
		private const string cHistory = "/history";
		// Разделитель между командами и параметрами
		private const char delimiter = '|';
		// Количество последних команд в истории
		private const int historySize = 15;

		/// <summary>
		/// Подключаемся
		/// </summary>
		public void Start()
		{
			client = new TelegramBotClient(apikey);

			client.OnMessage += OnMessage;// Событие при получении сообщения 

			client.StartReceiving();

		}

		/// <summary>
		/// Получаем сообщение и обрабатываем его
		/// </summary>
		/// <msgArr name="sender"></msgArr>
		/// <msgArr name="e"></msgArr>
		async void OnMessage(Object sender, MessageEventArgs e)
		{
			var msgText = e.Message.Text;
			int tmp;

			if (msgText == null)
			{
				return;
			}
			string[] authorizedUsers = project.Variables["authorizedUsers"].Value.Split(',');
			string[] msgArr = msgText.Split(delimiter);
			long id = e.Message.Chat.Id;

			project.SendInfoToLog(e.Message.Chat.Username + " написал: " + msgText, true);
			AddHistory(msgText);
			if (!authorizedUsers.Contains(e.Message.Chat.Username))
			{
				SendMessage(id, "Недостаточно прав!");
				project.SendWarningToLog(e.Message.Chat.Username + " не авторизованный пользователь написал : " + msgText, true);
				return;
			}

			string templateName = msgArr.Length >= 2 ? msgArr[1] : "";
			string param = msgArr.Length >= 3 ? msgArr[2] : "0";

			bool isInt = Int32.TryParse(param, out tmp);

			Process process = new Process();
			process.StartInfo.FileName = tasksRunerPath;

			// Обработка команд
			switch (msgArr[0])
			{
				case cAll:
					ProjList(id, false);
					break;
				case cWorking:
					ProjList(id, true);
					break;
				case "/help":
					Help(id);
					break;
				case "/history":
					GetHistory(id);
					break;
				case cGetCpu: // Узнать загруженность системы
					float cpu;
					string freeMemery;
					//Процессора
					using (var pc = new System.Diagnostics.PerformanceCounter("Processor", "% Processor Time", "_Total"))
					{
						cpu = pc.NextValue();
						Thread.Sleep(1000);
						cpu = pc.NextValue();
					}
					// Свободно оперативы
					using (var mem = new System.Diagnostics.PerformanceCounter("Memory", "Available MBytes"))
					{
						freeMemery = (mem.NextValue() / 1024.0).ToString("N2"); ;
					}
					SendMessage(id, string.Format(@"Примерная нагрузка на CPU :{0}%. {2}Свободно памяти: {1} GB", (int)cpu, freeMemery, "\n"));
					break;
				case cAddTries:
					if (!isInt)
					{
						SendMessage(id, "Необходим числовой параметр");
						break;
					}
					process.StartInfo.Arguments = "-o AddTries " + param + @" -names """ + templateName + @"""";
					process.Start();
					SendMessage(id, string.Format(@"Добавлено <u>{0}</u> заданий к выполнению для ""<u>{1}</u>""", param, templateName));
					break;
				case cInterruptTask:
					process.StartInfo.Arguments = "-o InterruptTask " + @"-names """ + templateName + @"""";
					process.Start();
					SendMessage(id, string.Format(@"""<u>{0}</u>"" прерван", templateName));
					break;
				case cSetTries:
					if (!isInt)
					{
						SendMessage(id, "Необходим числовой параметр");
						break;
					}
					process.StartInfo.Arguments = "-o SetTries " + param + @" -names """ + templateName + @"""";
					process.Start();
					SendMessage(id, string.Format(@"Установлено <u>{0}</u> заданий к выполнению для ""<u>{1}</u>""", param, templateName));
					break;
				case cSetThreads:
					if (!isInt)
					{
						SendMessage(id, "Необходим числовой параметр");
						break;
					}
					process.StartInfo.Arguments = "-o SetThreads " + param + @" -names """ + templateName + @"""";
					process.Start();
					SendMessage(id, string.Format(@"Установлено <u>{0}</u> максимально потоков для ""<u>{1}</u>""", param, templateName));
					break;
				case cStartTask:
					process.StartInfo.Arguments = "-o StartTask " + @"-names """ + templateName + @"""";
					process.Start();
					SendMessage(id, string.Format(@"""<u>{0}</u>"" запущен", templateName));
					break;
				case cStopTask:
					process.StartInfo.Arguments = "-o StopTask " + @"-names """ + templateName + @"""";
					process.Start();
					SendMessage(id, string.Format(@"""<u>{0}</u>"" остановлен", templateName));
					break;
				default:
					UnknownCommand(id);
					break;
			}
			process.Dispose();
		}
		/// <summary>
		/// Отправляем сообщение
		/// </summary>
		/// <param name="id"></param>
		/// <param name="msg"></param>
		async void SendMessage(long id, string msg)
		{

			// Формируем кнопики
			var rkm = new ReplyKeyboardMarkup();
			rkm.Keyboard =
				new KeyboardButton[][]
				{
					new KeyboardButton[]
					{
						new KeyboardButton(cAll),
						new KeyboardButton(cWorking),
						new KeyboardButton(cHelp),
						new KeyboardButton(cHistory)

					}
				};
			// Отправляем сообщение

			await client.SendTextMessageAsync(id, msg, Telegram.Bot.Types.Enums.ParseMode.Html, replyMarkup: rkm).ConfigureAwait(false);

		}
		/// <summary>
		/// Формируем и оправляем список проектов
		/// </summary>
		/// <param name="id"></param>
		/// <param name="onlyWork">Только запущенные</param>
		public void ProjList(long id, bool onlyWork)
		{
			string msg = "";
			try
			{

				var allProj = ZTask.GetTasks(onlyWork);
				foreach (var t in allProj)
				{

					msg += "<b><u>Имя шаблона: </u></b>" + "<code> " + t.Value.name + "</code>" + " Статус: " + t.Value.status + " Успешных выполнений: " + t.Value.goodCount + " Сколько делать: " + t.Value.taskCount + " Максимум потоков: " + t.Value.maxThreads + " Неуспехов подряд: " + t.Value.failCount + "\n\n";
				}
				if (allProj.Count == 0)
				{
					msg = "Нет запущенных проектов";

				}
			}
			catch (Exception)
			{


				throw new Exception("wtf");
			}

			try
			{

				SendMessage(id, msg);

			}
			catch (Exception)
			{

				throw new Exception("wtf2");
			}

		}

		/// <summary>
		/// Вызываем справку
		/// </summary>
		/// <param name="id"></param>
		public void Help(long id)
		{
			SendMessage(id, "Справка");
			string helpMsg =
				string.Format(@"Команды выглядят следующим образом: "" команда|название шаблона|параметр"". 
				Параметр нужен не для всех действий. ""|"" является разделителем. Изменить его можно в файле ""tgbot.cs"" 
				в константе ""delimiter"". Используйте только символы которые не используются в названии шаблона и в названии команд.{0}
				Клик по названию шаблона в мобильной версии - скопировать название шаблона в буфер. Так же можно скопировать команды из {11}{0}
				Изменить команды можно в файле ""tgbot.cs"". В самом начале достаточно поменять значения подписанных констант.{0}
				Примеры команд:{0}
				""<code>{2}{1}названеешаблона{1}17</code>"" - добавит указанному шаблону 17 выполнений{0}
				""<code>{3}{1}названеешаблона</code>"" - Прервет выполнение указанного шаблона{0}
				""<code>{4}{1}названеешаблона{1}24</code>"" - Установит 24 попытки выполнения{0}
				""<code>{5}{1}названеешаблона{1}12</code>"" - Установит 12 потоков{0}
				""<code>{6}{1}названеешаблона</code>"" - Запустит шаблон{0}
				""<code>{7}{1}названеешаблона</code>"" - Остановит шаблон{0}
				""<code>{8}</code>"" - Покажет текущую нагрузку на систему{0}
				""<code>{9}</code>"" - Выведет все шаблоны и информацию о них{0}
				""<code>{10}</code>"" - Выведет все запущенные шаблоны и информацию о них{0}	
				""<code>{11}</code>"" - Выведет последние отправленные команды",
				"\n", delimiter, cAddTries, cInterruptTask, cSetTries, cSetThreads, cStartTask, cStopTask, cGetCpu, cAll, cWorking, cHistory);

			SendMessage(id, helpMsg);
		}

		/// <summary>
		/// Обработка неизвестной команды
		/// </summary>
		/// <param name="id"></param>
		public void UnknownCommand(long id)
		{
			project.SendInfoToLog("UnknownCommand", true);
			SendMessage(id, "Неправильная команда");
		}

		/// <summary>
		/// Добавляет логи в историю команд
		/// </summary>
		/// <param name="message"></param>
		public void AddHistory(string message)
		{
			lock (tgLock)
			{
				var historylist = project.Lists["history"];
				if (!historylist.Contains(message))
				{
					historylist.Add(message);
					while (historylist.Count > historySize)
					{
						historylist.RemoveAt(0);
					}
				}
			}
		}
		/// <summary>
		/// Выводим историю
		/// </summary>
		/// <param name="id"></param>
		public void GetHistory(long id)
		{
			lock (tgLock)
			{
				var historylist = project.Lists["history"];
				string message = "";
				foreach (var s in project.Lists["history"])
				{
					message += "<code>" + s + @"</code>" + "\n";
				}
				SendMessage(id, message);
			}
		}
	}

	public class ZTask
	{
		public string name;
		public string goodCount;
		public string maxThreads;
		public string failCount;
		public string status;
		public string taskCount;

		public static Dictionary<string, ZTask> TaskArr;

		public ZTask(string name, string goodCount, string taskCount, string maxThreads, string failCount, string status)
		{
			this.name = name;
			this.goodCount = goodCount;
			this.maxThreads = maxThreads;
			this.failCount = failCount;
			this.taskCount = taskCount;
			this.status = status;

		}

		/// <summary>
		/// Парсим список шаблонов и информацию о них
		/// </summary>
		/// <param name="onlyWork"></param>
		/// <returns></returns>
		public static Dictionary<string, ZTask> GetTasks(bool onlyWork)
		{
			Dictionary<string, ZTask> result = new Dictionary<string, ZTask>();
			var tasks = ZennoPoster.TasksList;
			foreach (var t in tasks)
			{

				var doc = new System.Xml.XmlDocument();
				doc.LoadXml("<Task>" + t + "</Task>");
				string name = doc.SelectSingleNode("Task/Name").InnerXml;
				string goodCount = "";
				try
				{
					goodCount = doc.SelectSingleNode("Task/ExecutionSettings/DoneSuccesfully").InnerXml;

				}
				catch (Exception)
				{

					goodCount = doc.SelectSingleNode("Task/ExecutionSettings/DoneSuccessfully").InnerXml;

				}
				string taskCount = doc.SelectSingleNode("Task/ExecutionSettings/NumberOfTries").InnerXml;
				string maxThreads = doc.SelectSingleNode("Task/ExecutionSettings/LimitOfThreads").InnerXml;
				string failCount = doc.SelectSingleNode("Task/ExecutionSettings/NumOfFailStop").InnerXml;
				string status = doc.SelectSingleNode("Task/ExecutionSettings/Status").InnerXml;

				if (!onlyWork)
				{
					result.Add(name, new ZTask(name, goodCount, taskCount, maxThreads, failCount, status));
				}
				else
				if (status == "Perform")
				{
					result.Add(name, new ZTask(name, goodCount, taskCount, maxThreads, failCount, status));
				}
				else
				{

				}
			}

			return result;
		}

	}
}
